// Title:   ScanTools.h
// Created: Thu Sep 25 15:15:44 2003
// Authors: Simon Heinzle, Tim Weyrich
//
// Copyright (c) 2001, 2002, 2003, Computer Graphics Lab, ETH Zurich
//
// This file is part of the Pointshop3D system.
// See http://www.pointshop3d.com/ for more information.
//
// This program is free software; you can redistribute it and/or
// modify it under the terms of the GNU Lesser General Public
// License as published by the Free Software Foundation; either
// version 2.1 of the License, or (at your option) any later
// version.
// 
// This program is distributed in the hope that it will be useful,
// but WITHOUT ANY WARRANTY; without even the implied warranty of
// MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
// GNU Lesser General Public License for more details.
// 
// You should have received a copy of the GNU Lesser General
// Public License along with this library; if not, write to the
// Free Software Foundation, Inc., 59 Temple Place, Suite 330,
// Boston, MA 02111-1307 USA
// 
// Contact info@pointshop3d.com if any conditions of this
// licensing are not clear to you.
//

#ifndef __SCANTOOLS_H_
#define __SCANTOOLS_H_

// **********************************************
// To compile plugins (win32):
// 
// define TOOL_EXPORTS on the command line
//
// in Visual Studio:
// Project->Settings->C/C++, Category Preprocessor
// add TOOL_EXPORTS to 'preprocessor definitions'
// ***********************************************

#include <qtoolbutton.h>
#include <qtoolbar.h>
#include <qcursor.h>
#include <qevent.h>
#include <qwidget.h>
#include <qobject.h>
#include <qpopupmenu.h>

#include "../../../../Core/Interfaces/src/ToolInterface.h"
#include "ScanToolsDialogImpl.h"
#include "ScanToolsConfigurationDialogImpl.h"
#include "ScanToolsBoxColorDialogImpl.h"

// *******
// * tool includes
#include "BoxSelector.h"
#include "ActionManager.h"

/**
 * This class implements the Scan Tools!
 *
 *
 * @author Simon Heinzle
 * @version 0.0
 */
class ScanTools : public ToolInterface {
	
	Q_OBJECT


public:
	
	TOOL_API ScanTools();
	TOOL_API virtual ~ScanTools();

	// ***********************
	// interface ToolInterface
	// ***********************

	TOOL_API QToolButton* addButtonToToolBar (QToolBar *toolBar);
	TOOL_API QToolButton *getToolButton();
	TOOL_API QCursor *getToolCursor() const;
	TOOL_API QWidget *getToolConfigurationWidget() const;
	TOOL_API bool isAvailable() const;
	
	TOOL_API void handleMousePressEvent   (QMouseEvent *me);
	TOOL_API void handleMouseReleaseEvent (QMouseEvent *me);
	TOOL_API void handleMouseMoveEvent    (QMouseEvent *me);
	TOOL_API void handleKeyPressEvent     (QKeyEvent   *ke);
	TOOL_API void handleKeyReleaseEvent   (QKeyEvent   *ke);

	TOOL_API void selectTool();
	TOOL_API void deselectTool();

public slots:
	// mouse events
	void handleKeyPressEventFromDialog(QKeyEvent* ke);
	void handleKeyReleaseEventFromDialog(QKeyEvent* ke);
	void handleRendererWidgetRepainted();
	
	// called whenever new tool is selected
	void handleToolSelected(int id);

	// slots when objects change
	void handleObjectAdded();
	void handleObjectRemoved();
	
	// selection options slot
	void handleShapeBoxClicked(int id);
	void handleSelectAllButtonReleased();
	void handleDeselectButtonReleased();
	void handleObjectListSelectionChanged();
	void handleMouseBoxClicked(int id);
	
	// deletion slots
	void handleDeleteButtonReleased();
	// random noise slots
	void handleRandomNoiseButtonReleased();
	// mls slots
	void handleMLSButtonReleased();
	// particle slots
	void handlePerformParticleButtonReleased();
	// outlier slots
	void handleTabWidgetSelected(const QString& str);
	void handleSkipRebuildButtonReleased();
	void handleALLClassifyButtonReleased();
	void handleOutlierPreviewButtonReleased();
	void handleRestoreButtonReleased();
	void handleOutlierDeletionButtonReleased();
	void handleBuildButtonReleased();
	void handleCombinerMethod1SliderMoved(int value);
	void handleCombinerMethod2SliderMoved(int value);
	void handleCombinerMethod3SliderMoved(int value);
	void handleAutomaticalBoxToggled(bool toggle);

	// additional outlier slots for easier gui handling
	void handleKNextEditReturn();
	void handleLEditReturn();
	void handleOutlierPercentageReturn();
	void handleCombinerMBReturn();
	void handleCombinerBIReturn();
	void handleCombinerPFReturn();

	// false normals
	void handleRemoveFalseNormalsButtonReleased();
	// resize
	void handleResizeButtonReleased();
	// recolor
	void handleDoColorButtonReleased();

	void handleMemLeak();

	// help buttons
	void handleHelp(const QString& helpstring);
	void handleHelpSelection ();
	void handleHelpDeletor ();
	void handleHelpNoise ();
	void handleHelpMLS ();
	void handleHelpScatterer ();
	void handleHelpLPS ();
	void handleHelpOutliers ();
	void handleHelpNormals ();
	void handleHelpResize ();
	void handleHelpRecolor ();

	// quick buttons
	void handleQuickMarkButtonReleased();
	void handleQuickDeleteButtonReleased();
	void handleQuickNoiseButtonReleased();
	void handleQuickMLSButtonReleased();
	void handleQuickScatterButtonReleased();
	void handleQuickLPSButtonReleased();
	void handleQuickOutlierButtonReleased();
	void handleQuickNormalButtonReleased();
	void handleQuickResizeButtonReleased();
	void handleQuickRecolorButtonReleased();
		
private:


	QCursor							*toolCursor;
	QToolButton						*toolButton;
	ScanToolsWidget					*scantoolsWidget;
	ScanToolsDialogImpl				*scantoolsDialog;
	ScanToolsConfigurationDialogImpl *scantoolsConfigurationDialog;
	ScanToolsBoxColorDialogImpl		*scantoolsBoxColorDialog;


	QPopupMenu						scantoolsMenu;
	int								markID, deleteID, noiseID,
		                            mlsID, scatterID, colorID,
									outlierID, normalsID, resizeID,
									lpsID;
	int								currentTool;
	QIconSet markSet,
		     deleteSet,
			 noiseSet,
			 mlsSet,
			 scatterSet,
			 colorSet,
			 outlierSet,
			 normalsSet,
			 resizeSet,
			 lpsSet;
	bool isToolSetSelected;
	// QVector for current objects
	QVector<Object> objects;
	// undo stack
	ActionManager *undoRedoStack;
	// Outlier Classificator
	OutlierClassificator *ocl;
	float w1,w2,w3;

	bool controlPressed;
	bool shiftPressed;
	bool altPressed;
	
	bool classified;

	void updateObjects();
	bool checkToolAvailability() const;
	void updateWindow();
	void showGL();
	void hideGL();
	int stateToInt(ButtonState state);

	void connectSelectionOptions();
	void disconnectSelectionOptions();
	void connectOther();
	void disconnectOther();
	int getCurrentSelectionIndex();

private slots:
	void handleToolButtonDialog(void);

};

// ****************
// access functions
// ****************

// extern "C" is needed so those functions can be accessed
// with GetProcAddress() (WIN32) and dlsym() (UNIX) respective
extern "C" {
	TOOL_API ToolInterface *createTool();
	TOOL_API const char *getToolName();
}

#endif  // __SCANTOOLS_H_

// Some Emacs-Hints -- please don't remove:
//
//  Local Variables:
//  mode:C++
//  tab-width:4
//  End:
