// Title:   cQuaternion.cpp
// Created: Thu Sep 25 15:14:50 2003
// Authors: Simon Heinzle, Tim Weyrich
//
// Copyright (c) 2001, 2002, 2003, Computer Graphics Lab, ETH Zurich
//
// This file is part of the Pointshop3D system.
// See http://www.pointshop3d.com/ for more information.
//
// This program is free software; you can redistribute it and/or
// modify it under the terms of the GNU Lesser General Public
// License as published by the Free Software Foundation; either
// version 2.1 of the License, or (at your option) any later
// version.
// 
// This program is distributed in the hope that it will be useful,
// but WITHOUT ANY WARRANTY; without even the implied warranty of
// MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
// GNU Lesser General Public License for more details.
// 
// You should have received a copy of the GNU Lesser General
// Public License along with this library; if not, write to the
// Free Software Foundation, Inc., 59 Temple Place, Suite 330,
// Boston, MA 02111-1307 USA
// 
// Contact info@pointshop3d.com if any conditions of this
// licensing are not clear to you.
//

#include <math.h>

/* Some <math.h> files do not define M_PI... (including VC) */
#ifndef M_PI
#define M_PI 3.14159265358979323846
#endif

#include "cQuaternion.h"

float cQuaternion::dot3(float *a, float *b) {
	return a[0]*b[0]+a[1]*b[1]+a[2]*b[2];
}

void cQuaternion::cross3(float *a, float *b, float *res) {
	res[0] = a[1]*b[2] - a[2]*b[1];
	res[1] = a[2]*b[0] - a[0]*b[2];
	res[2] = a[0]*b[1] - a[1]*b[0];
}

void cQuaternion::scalmult3(float a, float *b, float *res) {
	res[0] = a* b[0];
	res[1] = a* b[1];
	res[2] = a* b[2];
}

void cQuaternion::add3(float *a, float *b, float *res) {
	res[0] = a[0] + b[0];
	res[1] = a[1] + b[1];
	res[2] = a[2] + b[2];
}

void cQuaternion::normalize3(float *a) {
	float len;
	len = a[0]*a[0]+a[1]*a[1]+a[2]*a[2];
	len = (float) sqrt(len);
	a[0] /= len;
	a[1] /= len;
	a[2] /= len;
}

	
cQuaternion::cQuaternion(float x, float y, float z, float c) {
	_elements[0] = x;
	_elements[1] = y;
	_elements[2] = z;
	_elements[3] = c;
}

cQuaternion cQuaternion::operator+= (cQuaternion c) {
	_elements[0] += c._elements[0];
	_elements[1] += c._elements[1];
	_elements[2] += c._elements[2];
	_elements[3] += c._elements[3];
	return *this;
}

cQuaternion cQuaternion::operator+ (cQuaternion c) {
	cQuaternion tmp;
	tmp._elements[0] = c._elements[0]+_elements[0];
	tmp._elements[1] = c._elements[1]+_elements[1];
	tmp._elements[2] = c._elements[2]+_elements[2];
	tmp._elements[3] = c._elements[3]+_elements[3];
	return tmp;
}

cQuaternion cQuaternion::operator* (cQuaternion c) {
	float tmp1[4], tmp2[4];
	float res[4];
	
	scalmult3(_elements[3], c._elements, tmp1);
	scalmult3(c._elements[3], _elements, tmp2);
	add3(tmp1, tmp2, tmp2);
	cross3(_elements, c._elements, tmp1);
	add3(tmp1, tmp2, res);
	
	res[3] = _elements[3]*c._elements[3] - dot3(_elements, c._elements);	
	
	return cQuaternion(res[0],res[1],res[2],res[3]);
	
}

float& cQuaternion::operator[] (int index) {
	return _elements[index];
}

const float& cQuaternion::operator[] (int index) const {
	return _elements[index];
}

cQuaternion cQuaternion::getConjugate(void) {
	return cQuaternion((float)-1.0*_elements[0], (float)-1.0*_elements[1], (float)-1.0*_elements[2], (float)_elements[3]);
}

void cQuaternion::makeConjugate(void) {
	_elements[0] *= -1.0;
	_elements[1] *= -1.0;
	_elements[2] *= -1.0;
}

void cQuaternion::rotateRAD(float angle, float* xyz_axis) {
	float sin_angle = (float)sin(angle/2.0);
	cQuaternion q((float)sin_angle*xyz_axis[0],(float)sin_angle*xyz_axis[1],(float)sin_angle*xyz_axis[2],(float)cos(angle/2.0));
			
	*this = q * (*this) * q.getConjugate();
}

void cQuaternion::rotateDEG(float angle, float* xyz_axis) {
	angle =(float) (M_PI / 180.0 * angle);
	rotateRAD(angle, xyz_axis);
}

void cQuaternion::rotateRAD(float angle, cQuaternion xyz_quat) {
	float sin_angle =(float) sin(angle/2.0);
	
	normalize3(&xyz_quat[0]);
	xyz_quat[0] *= sin_angle;
	xyz_quat[1] *= sin_angle;
	xyz_quat[2] *= sin_angle;
	xyz_quat[3] = (float)cos(angle/2.0);
	
	*this = xyz_quat * (*this) * xyz_quat.getConjugate();
}

void cQuaternion::rotateDEG(float angle, cQuaternion xyz_quat) {
	angle = (float) (M_PI / 180.0 * angle);
	rotateRAD(angle, xyz_quat);
}
	
float* cQuaternion::getElements(void) {
	return _elements;
}

void cQuaternion::getRotationMatrixRAD(float angle, cQuaternion quat, float matrix[4][4]) {
	float sin_angle = (float)sin(angle/2.0);
	
	normalize3(&quat[0]);
	quat[0] *= sin_angle;
	quat[1] *= sin_angle;
	quat[2] *= sin_angle;
	quat[3] = (float)cos(angle/2.0);
	
	matrix[0][0] =(float) (1.0 - 2.0 * (quat[1]*quat[1] + quat[2]*quat[2]));
	matrix[1][0] =(float)       2.0 * (quat[0]*quat[1] - quat[2]*quat[3]);
	matrix[2][0] =(float)       2.0 * (quat[2]*quat[0] + quat[1]*quat[3]);
	matrix[3][0] =(float) 0.0;

	matrix[0][1] =(float)       2.0 * (quat[0]*quat[1] + quat[2]*quat[3]);
	matrix[1][1] =(float) (1.0 - 2.0 * (quat[2]*quat[2] + quat[0]*quat[0]));
	matrix[2][1] =(float)       2.0 * (quat[1]*quat[2] - quat[0]*quat[3]);
	matrix[3][1] =(float) 0.0;

	matrix[0][2] =(float)       2.0 * (quat[2]*quat[0] - quat[1]*quat[3]);
	matrix[1][2] =(float)       2.0 * (quat[1]*quat[2] + quat[0]*quat[3]);
	matrix[2][2] =(float) (1.0 - 2.0 * (quat[1]*quat[1] + quat[0]*quat[0]));
	matrix[3][2] =(float) 0.0;

	matrix[0][3] =(float) 0.0;
	matrix[1][3] =(float) 0.0;
	matrix[2][3] =(float) 0.0;
	matrix[3][3] =(float) 1.0;
}
 
void cQuaternion::getRotationMatrixDEG(float angle, cQuaternion quat, float matrix[4][4]) {
	angle = (float)(M_PI / 180.0 * angle);
	getRotationMatrixRAD(angle, quat, matrix);
}

// Some Emacs-Hints -- please don't remove:
//
//  Local Variables:
//  mode:C++
//  tab-width:4
//  End:
